@@ -1,11 +1,11 @@
 /* LayerObject.h
  * Object managing a single layer and its attributes
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * Created:  1996-03-07
- * Modified: 2005-11-06 (uniqueId)
+ * Modified: 2012-01-25 (-setInvisible:, -invisible, invisible)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -32,24 +32,30 @@
 #include "GraphicObjects.subproj/PerformanceMap.h"
 
 /* FIXME: CAM (should go to a sub class header) */
+#define OUTPUT_STEPMAX 100
 typedef struct
 {
-    float	step[100];
+    float	step[OUTPUT_STEPMAX];
     int		count;
-}OutputSteps;
+} OutputSteps;
 
-#define CUT_INSIDE	0
-#define CUT_OUTSIDE	1
-#define CUT_NOSIDE	2
-#define CUT_PICKOUT	3
-#define CUT_ISOLATION	4	// calculate isolation of PCB
-#define CUT_BLOWUP	5	// calculate blow up of PCB
-#define CUT_RUBOUT	6	// calculate rubout of PCB
+typedef enum
+{
+    CUT_INSIDE    = 0,
+    CUT_OUTSIDE   = 1,
+    CUT_NOSIDE    = 2,
+    CUT_PICKOUT   = 3,
+    CUT_ISOLATION = 4,	// calculate isolation of PCB
+    CUT_BLOWUP    = 5,	// calculate blow up   of PCB
+    CUT_RUBOUT    = 6	// calculate rubout    of PCB
+} CAMCutType;
 
 /* the layer ids shouldn't be changed, they are saved to the document !
+ * FIXME: we should save the name of the layer types (not integer code)
  */
 typedef enum
 {
+    /* General layer types */
     LAYER_STANDARD   = 0,	// standard layer
     LAYER_PASSIVE    = 1,	// passive layer, not editable, calculated automatically (CAM)
     LAYER_CLIPPING   = 2,	// layer with regions used for clipping (CAM)
@@ -57,52 +63,59 @@ typedef enum
     LAYER_FITTING    = 4,	// a layer with two marks for flipping the working piece (CAM)
     LAYER_PAGE       = 5,	// page in multi page document
 
+    /* Templates */
     LAYER_TEMPLATE   = 10,	// general template for all layers and pages
     LAYER_TEMPLATE_1 = 11,	// template for odd pages
-    LAYER_TEMPLATE_2 = 12	// template for even pages
+    LAYER_TEMPLATE_2 = 12,	// template for even pages
+
+    /* CAM (FIXME: needs to go to CAM module) */
+    LAYER_CAMERA     = 50	// layer with markers defining target net for camera gauging (CAM)
 } LayerType;
 
 @interface LayerObject: NSObject
 {
-    NSString		*string;		// the name of the layer
-    PerformanceMap	*performanceMap;	// grid for faster access of objects on layer
-    NSMutableArray	*list;			// the graphic list
-    int			tag;			// tag for layers with special purpose
-    NSColor 		*color;			// the color of this layer
-    int			state;			// if we have to display this layer
-    BOOL		editable;		// whether we are editable
-    int			type;			// type of layer: LAYER_STANDARD, LAYER_PASSIVE, ...
-    int			uniqueId;		// a unique id of the layer used with multi layer groups
-    BOOL		likeOtherLayers;	// YES = share parameters with other layers of state YES
-    BOOL		useForTile;		// whether layer is used for duplicates
-    BOOL		dirty;			// YES when changed
+    NSString        *string;            // the name of the layer
+    PerformanceMap  *performanceMap;    // grid for faster access of objects on layer
+    NSMutableArray  *list;              // the graphic list
+    int             tag;                // tag for layers with special purpose
+    NSColor         *color;             // the color of this layer
+    int             state;              // if we have to display this layer
+    BOOL            editable;           // whether we are editable
+    int             type;               // type of layer: LAYER_STANDARD, LAYER_PASSIVE, ...
+    int             uniqueId;           // a unique id of the layer used with multi layer groups
+    BOOL            likeOtherLayers;    // YES = share parameters with other layers of state YES
+    BOOL            useForTile;         // whether layer is used for duplicates
+    BOOL            dirty;              // YES when changed
+    BOOL            invisible;          // whether this is a drawable layer
+
+    NSMutableDictionary *layerDict;     // dictionary to store additional stuff for the layer
 
     /* FIXME: CAM (should go to a sub class) */
-    int			toolIndex;		// the index of the tool
-    NSString		*toolString;		// the name of the tool
-    BOOL		filled;			// whether we are filled
-    BOOL		mirrored;		// whether we are mirrored
-    int			side;			// whether to cut inside, outside or on the line
-    float		flatness;
-    float		fillOverlap;		// (0.0 - 1.0) 0 = no overlap
-    float		fillDirection;		// (0.0 - 360.0)
-    BOOL		removeLoops;		// whether we have to remove garbage (loops) from contour
-    BOOL		revertDirection;	// whether we revert the direction (ccw/cw)
-    float		dippingDepth;		// the dippingDepth [mm]
-    float		approachAngle;		// the approach angle for dipping, 0 - 89 [deg]
-    BOOL		stepwise;		// weather stepwise cutting is active
-    int			numSteps;		// number of middle steps
-    float		step[3];		// first, step width, final step [mm]
-    OutputSteps		calcSteps;		// calculated steps [mm]
-    float		settle;			// [mm]
-    BOOL		settleBefore;		// settlement before last step z
-    int			levelingX, levelingY;	// number of test points for leveling */
-    BOOL		inlay;
+    int             toolIndex;          // the index of the tool
+    NSString        *toolString;        // the name of the tool
+    BOOL            filled;             // whether we are filled
+    BOOL            mirrored;           // whether we are mirrored
+    int             side;               // whether to cut inside, outside or on the line
+    float           flatness;
+    float           fillOverlap;        // (0.0 - 1.0) 0 = no overlap
+    float           fillDirection;      // (0.0 - 360.0)
+    BOOL            removeLoops;        // whether we have to remove garbage (loops) from contour
+    BOOL            revertDirection;    // whether we revert the direction (ccw/cw)
+    float           dippingDepth;       // the dippingDepth [mm]
+    float           approachAngle;      // the approach angle for dipping, 0 - 89 [deg]
+    BOOL            stepwise;           // weather stepwise cutting is active
+    int             numSteps;           // number of middle steps
+    float           step[3];            // first, step width, final step [mm]
+    OutputSteps     calcSteps;          // calculated steps [mm]
+    float           settle;             // [mm]
+    BOOL            settleBefore;       // settlement before last step z
+    int             levelingX, levelingY;   // number of test points for leveling */
+    BOOL            inlay;
 
     /* CAM output */
-    NSArray		*webList;		// used to reach web list when creating output
-    int			tileCount;		// used to reach tile count when creating output
-    id			clipObject;		// used to reach clip object when creating output
+    NSArray         *webList;           // used to reach web list when creating output
+    int             tileCount;          // used to reach tile count when creating output
+    id              clipObject;         // used to reach clip object when creating output
 }
 
 + layerObject;
@@ -140,6 +153,9 @@ typedef enum
 - (void)setState:(int)flag;
 - (int)state;
 
+- (void)setInvisible:(int)flag;
+- (int)invisible;
+
 - (void)setEditable:(BOOL)flag;
 - (BOOL)editable;
 
