@@ -1,11 +1,13 @@
 /* LayerObject.m
  * Object managing a single layer and its attributes
  *
- * Copyright (C) 1996-2003 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * Created:  1996-03-07
- * Modified: 2003-06-26
+ * Modified: 2012-01-25 (-setInvisible:, -invisible)
+ *           2012-01-24 (-stepsForDip:)
+ *           2008-11-12 (layerDict, clean-up)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -60,29 +62,31 @@
 
     [self setString:UNTITLED_STRING];
     [self setToolString:@""];
-    state = YES;
-    editable = YES;
-    useForTile = YES;
+    state           = YES;
+    editable        = YES;
+    uniqueId        = 0;
+    useForTile      = YES;
     likeOtherLayers = YES;
-    dirty = NO;
-    list = [NSMutableArray new];	// the object list
+    dirty           = NO;
+    list            = [NSMutableArray new]; // the object list
 
     /* CAM */
-    toolIndex = -1;
-    filled = NO;
-    mirrored = NO;
-    inlay = NO;
-    side = CUT_NOSIDE;
-    flatness = 0.15;
-    removeLoops = YES;
+    toolIndex       = -1;
+    filled          = NO;
+    mirrored        = NO;
+    inlay           = NO;
+    side            = CUT_NOSIDE;
+    flatness        = 0.15;
+    removeLoops     = YES;
     revertDirection = NO;
-    fillOverlap = 0.25;
-    fillDirection = 0.0;
-    stepwise = NO;
+    fillOverlap     = 0.15;		// 15%
+    fillDirection   = 0.0;
+    stepwise        = NO;
     calcSteps.count = 0;
-    settleBefore = NO;
-    inlay = NO;
-    levelingX = levelingY = 2;
+    settleBefore    = NO;
+    inlay           = NO;
+    levelingX       = levelingY = 2;
+    invisible       = NO;
 
     return self;
 }
@@ -135,6 +139,14 @@
     [performanceMap addObject:obj];
     dirty = YES;
 }
+/* faster without check if object already is in list !
+ */
+- (void)addObjectWithoutCheck:(id)obj
+{
+    [list addObject:obj];
+    [performanceMap addObject:obj];
+    dirty = YES;
+}
 - (void)addObjectsFromArray:(NSArray*)array
 {
     [list addObjectsFromArray:array];
@@ -214,17 +226,20 @@
     return NO;
 }
 
-- (void)setString:(NSString *)aString	{ [string release]; string = [aString retain]; }
-- (NSString *)string			{ return string; }
+- (void)setString:(NSString *)aString   { [string release]; string = [aString retain]; }
+- (NSString *)string                    { return string; }
 
-- (void)setTag:(int)newTag		{ tag = newTag; }
-- (int)tag				{ return tag; }
+- (void)setTag:(int)newTag              { tag = newTag; }
+- (int)tag                              { return tag; }
 
-- (void)setColor:(NSColor *)aColor	{ color=aColor; }
-- (NSColor *)color			{ return color; }
+- (void)setColor:(NSColor *)aColor      { color=aColor; }
+- (NSColor *)color                      { return color; }
 
-- (void)setState:(int)flag		{ state = flag; }
-- (int)state				{ return state; }
+- (void)setState:(int)flag              { state = flag; }
+- (int)state                            { return state; }
+
+- (void)setInvisible:(int)flag          { invisible = flag; }
+- (int)invisible                        { return invisible; }
 
 - (void)setEditable:(BOOL)flag
 {
@@ -238,12 +253,23 @@
 - (void)setType:(int)newType
 {
     type = newType;
-    if (type == LAYER_CLIPPING || type == LAYER_LEVELING)
-    {   likeOtherLayers = NO;
-        useForTile = NO;
+    switch (type)
+    {
+        case LAYER_CLIPPING:
+        case LAYER_LEVELING:
+        case LAYER_TEMPLATE:
+        case LAYER_TEMPLATE_1:
+        case LAYER_TEMPLATE_2:
+            likeOtherLayers = NO;
+            useForTile = NO;
+        default:
+            break;
     }
 }
 
+- (int)uniqueId				{ return uniqueId; }
+- (void)setUniqueId:(int)newId		{ uniqueId = newId; }
+
 - (BOOL)useForTile		{ return (type == LAYER_CLIPPING || type == LAYER_LEVELING) ? NO : useForTile; }
 - (void)setUseForTile:(BOOL)flag	{ useForTile = flag;  }
 - (void)setTileCount:(int)c		{ tileCount = c; }
@@ -276,15 +302,15 @@
 /*
  * CAM methods
  */
-- (void)setToolIndex:(int)aTool		{ toolIndex = aTool; }
-- (int)toolIndex			{ return toolIndex; }
+- (void)setToolIndex:(int)aTool { toolIndex = aTool; }
+- (int)toolIndex                { return toolIndex; }
 
 - (void)setToolString:(NSString*)aString
 {
     [toolString release];
     toolString = (!aString) ? @"" : [aString retain];
 }
-- (NSString*)toolString			{ return toolString; }
+- (NSString*)toolString         { return toolString; }
 
 - (void)setSide:(int)s
 {
@@ -458,7 +484,7 @@
         else
         {   float	h = dippingDepth - step[0] - step[2];
 
-            numSteps = Max(1, h/height);
+            numSteps = Min( Max(1, h/height), OUTPUT_STEPMAX-1);
             step[1] = h/numSteps;
         }
         calcSteps.count = 0;
@@ -516,16 +542,17 @@
         calcSteps = [self stepsForDip:dippingDepth];
     return calcSteps;
 }
-- (OutputSteps)stepsForDip:(float)dip
-{   OutputSteps	dipSteps;
-    float	h, prev = 0.0;
-    int		i;
+- (OutputSteps)stepsForDip:(float)dip   // [mm]
+{   OutputSteps dipSteps;
+    float       h, prev = 0.0;
+    int         i;
 
     dipSteps.count = 0;
 
     if (!stepwise || dip == 0.0)
     {   dipSteps.count = 1;
-        dipSteps.step[0] = -dippingDepth;
+        //dipSteps.step[0] = -dippingDepth;
+        dipSteps.step[0] = -dip;    // 2012-01-24
         return dipSteps;
     }
 
@@ -533,7 +560,8 @@
     {   dipSteps.step[0] = prev = -step[0];
         dipSteps.count ++;
     }
-    h = dippingDepth - step[0] - step[2];
+    //h = dippingDepth - step[0] - step[2];
+    h = dip - step[0] - step[2];    // 2012-01-24
     if (h > TOLERANCE)
         for (i=0; i<numSteps; i++)
         {   dipSteps.step[dipSteps.count] = prev = prev - h/numSteps;
@@ -581,7 +609,7 @@
 - clipObject				{ return clipObject; }
 
 
-/* Archiving methods
+/* Archiving methods (DEPRECATED - Property list is used)
  */
 - (void)encodeWithCoder:(NSCoder *)aCoder
 {
@@ -609,19 +637,19 @@
     [aDecoder decodeValuesOfObjCTypes:"icc", &state, &filled, &editable];
     if ( version < 1 )
         [aDecoder decodeValuesOfObjCTypes:"cc", &removeLoops, &flag];
-    else if ( version < 3 ) // 03.98
+    else if ( version < 3 )	// 03.98
     {   [aDecoder decodeValuesOfObjCTypes:"cc", &removeLoops, &flag];
         type = (flag) ? LAYER_PASSIVE : LAYER_STANDARD;
     }
-    else // 07.12.99
+    else 			// 07.12.99
     {   [aDecoder decodeValuesOfObjCTypes:"ci", &removeLoops, &type];
-        [aDecoder decodeValuesOfObjCTypes:"ii", &levelingX, &levelingY];	/* 11.12.99 */
+        [aDecoder decodeValuesOfObjCTypes:"ii", &levelingX, &levelingY];	// 11.12.99
     }
-    if ( version >= 2 )	// 07.04.98
+    if ( version >= 2 )		// 07.04.98
         [aDecoder decodeValuesOfObjCTypes:"c", &revertDirection];
     [aDecoder decodeValuesOfObjCTypes:"i", &side];
     [aDecoder decodeValuesOfObjCTypes:"f", &dippingDepth];
-    if (version >= 4)	// 2000-09-27
+    if (version >= 4)		// 2000-09-27
         [aDecoder decodeValuesOfObjCTypes:"ff", &flatness, &fillOverlap];
     else
     {   [aDecoder decodeValuesOfObjCTypes:"ffff", &flatness, &minLineLength, &maxLineLength, &fillOverlap];
@@ -638,91 +666,103 @@
  */
 - (id)propertyList
 {   NSMutableDictionary *plist = [NSMutableDictionary dictionaryWithCapacity:9];
-    NSPoint		p;
+    NSPoint             p;
 
-    [plist setObject:NSStringFromClass([self class]) forKey:@"Class"];
-    [plist setObject:propertyListFromInt([[self class] version]) forKey:@"v"];
-    [plist setObject:string forKey:@"string"];
-    [plist setInt:tag forKey:@"tag"];
-    [plist setObject:propertyListFromInt(state) forKey:@"state"];
-    if (editable) [plist setObject:@"YES" forKey:@"editable"];
-    [plist setObject:propertyListFromInt(type) forKey:@"type"];
-    if (useForTile) [plist setObject:@"YES" forKey:@"useForTile"];
+    [plist setObject:NSStringFromClass([self class])                forKey:@"Class"];
+    [plist setObject:propertyListFromInt([[self class] version])    forKey:@"v"];
+    [plist setObject:string                                         forKey:@"string"];
+    [plist setInt:tag                                               forKey:@"tag"];
+    [plist setObject:propertyListFromInt(state)                     forKey:@"state"];
+    if (editable) [plist setObject:@"YES"                           forKey:@"editable"];
+    [plist setObject:propertyListFromInt(type)                      forKey:@"type"];
+    if (uniqueId)
+        [plist setObject:propertyListFromInt(uniqueId)              forKey:@"uniqueId"];
+    if (useForTile) [plist setObject:@"YES"                         forKey:@"useForTile"];
     if ( color )
-        [plist setObject:propertyListFromNSColor(color) forKey:@"color"];
-    [plist setObject:propertyListFromArray(list) forKey:@"list"];
+        [plist setObject:propertyListFromNSColor(color)             forKey:@"color"];
+    [plist setObject:propertyListFromArray(list)                    forKey:@"list"];
+
+    if (layerDict)
+        [plist setObject:layerDict                                  forKey:@"layerDict"];
 
     /* CAM */
-    [plist setObject:propertyListFromInt(toolIndex) forKey:@"toolIndex"];
-    [plist setObject:toolString forKey:@"toolString"];
-    if (filled) [plist setObject:@"YES" forKey:@"filled"];
-    if (mirrored) [plist setObject:@"YES" forKey:@"mirrored"];
-    if (removeLoops) [plist setObject:@"YES" forKey:@"removeLoops"];
+    [plist setObject:propertyListFromInt(toolIndex)                 forKey:@"toolIndex"];
+    [plist setObject:toolString                                     forKey:@"toolString"];
+    if (filled) [plist setObject:@"YES"                             forKey:@"filled"];
+    if (mirrored) [plist setObject:@"YES"                           forKey:@"mirrored"];
+    if (removeLoops) [plist setObject:@"YES"                        forKey:@"removeLoops"];
     p = NSMakePoint((float)levelingX, (float)levelingY);
-    [plist setObject:propertyListFromNSPoint(p) forKey:@"levelingLimits"];
-    if (revertDirection) [plist setObject:@"YES" forKey:@"revertDirection"];
-    [plist setObject:propertyListFromInt(side) forKey:@"side"];
-    [plist setObject:propertyListFromFloat(dippingDepth) forKey:@"dippingDepth"];
-    [plist setObject:propertyListFromFloat(approachAngle) forKey:@"approachAngle"];
-    [plist setObject:propertyListFromFloat(flatness) forKey:@"flatness"];
-    [plist setObject:propertyListFromFloat(fillOverlap) forKey:@"fillOverlap"];
-    [plist setObject:propertyListFromFloat(fillDirection) forKey:@"fillDirection"];
-    if (inlay) [plist setObject:@"YES" forKey:@"inlay"];
-    if (likeOtherLayers) [plist setObject:@"YES" forKey:@"likeOtherLayers"];
+    [plist setObject:propertyListFromNSPoint(p)                     forKey:@"levelingLimits"];
+    if (revertDirection) [plist setObject:@"YES"                    forKey:@"revertDirection"];
+    [plist setObject:propertyListFromInt(side)                      forKey:@"side"];
+    [plist setObject:propertyListFromFloat(dippingDepth)            forKey:@"dippingDepth"];
+    [plist setObject:propertyListFromFloat(approachAngle)           forKey:@"approachAngle"];
+    [plist setObject:propertyListFromFloat(flatness)                forKey:@"flatness"];
+    [plist setObject:propertyListFromFloat(fillOverlap)             forKey:@"fillOverlap"];
+    [plist setObject:propertyListFromFloat(fillDirection)           forKey:@"fillDirection"];
+    if (inlay) [plist setObject:@"YES"                              forKey:@"inlay"];
+    if (likeOtherLayers) [plist setObject:@"YES"                    forKey:@"likeOtherLayers"];
 
-    if (stepwise) [plist setObject:@"YES" forKey:@"stepwise"];
-    [plist setInt:numSteps forKey:@"numSteps"];
-    [plist setObject:propertyListFromFloat(step[0]) forKey:@"step0"];
-    [plist setObject:propertyListFromFloat(step[1]) forKey:@"step1"];
-    [plist setObject:propertyListFromFloat(step[2]) forKey:@"step2"];
+    if (stepwise) [plist setObject:@"YES"                           forKey:@"stepwise"];
+    [plist setInt:numSteps                                          forKey:@"numSteps"];
+    [plist setObject:propertyListFromFloat(step[0])                 forKey:@"step0"];
+    [plist setObject:propertyListFromFloat(step[1])                 forKey:@"step1"];
+    [plist setObject:propertyListFromFloat(step[2])                 forKey:@"step2"];
 
-    if (settleBefore) [plist setObject:@"YES" forKey:@"settleBefore"];
-    [plist setObject:propertyListFromFloat(settle) forKey:@"settle"];
+    if (settleBefore) [plist setObject:@"YES"                       forKey:@"settleBefore"];
+    [plist setObject:propertyListFromFloat(settle)                  forKey:@"settle"];
+
+    if (invisible) [plist setObject:@"YES"                          forKey:@"invisible"];
 
     return plist;
 }
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString *)directory
 {   NSPoint	p;
 
-    string = [[plist objectForKey:@"string"] retain];
-    tag = [plist intForKey:@"tag"];
-    state = [plist intForKey:@"state"];
-    editable = ([plist objectForKey:@"editable"] ? YES : NO);
-    type = [plist intForKey:@"type"];
+    string          = [[plist objectForKey:@"string"] retain];
+    tag             = [plist intForKey:@"tag"];
+    state           = [plist intForKey:@"state"];
+    editable        = ([plist objectForKey:@"editable"] ? YES : NO);
+    type            = [plist intForKey:@"type"];
+    uniqueId        = [plist intForKey:@"uniqueId"];
     likeOtherLayers = ([plist objectForKey:@"likeOtherLayers"] ? YES : NO);
-    useForTile = ([plist objectForKey:@"useForTile"] ? YES : NO);
-    color = colorFromPropertyList([plist objectForKey:@"color"], [self zone]);
-    list = arrayFromPropertyList([plist objectForKey:@"list"], directory, [self zone]);
+    useForTile      = ([plist objectForKey:@"useForTile"] ? YES : NO);
+    color           = colorFromPropertyList([plist objectForKey:@"color"], [self zone]);
+    list            = arrayFromPropertyList([plist objectForKey:@"list"], directory, [self zone]);
+
+    layerDict       = [[plist objectForKey:@"layerDict"] mutableCopy];
 
     /* CAM */
-    toolIndex = [plist intForKey:@"toolIndex"];
-    toolString = [[plist objectForKey:@"toolString"] retain];
-    filled = ([plist objectForKey:@"filled"] ? YES : NO);
-    mirrored = ([plist objectForKey:@"mirrored"] ? YES : NO);
-    removeLoops = ([plist objectForKey:@"removeLoops"] ? YES : NO);
-    p = pointFromPropertyList([plist objectForKey:@"levelingLimits"]);
-    levelingX = (int)p.x;
-    levelingY = (int)p.y;
+    toolIndex       = [plist intForKey:@"toolIndex"];
+    toolString      = [[plist objectForKey:@"toolString"] retain];
+    filled          = ([plist objectForKey:@"filled"] ? YES : NO);
+    mirrored        = ([plist objectForKey:@"mirrored"] ? YES : NO);
+    removeLoops     = ([plist objectForKey:@"removeLoops"] ? YES : NO);
+    p               = pointFromPropertyList([plist objectForKey:@"levelingLimits"]);
+    levelingX       = (int)p.x;
+    levelingY       = (int)p.y;
     revertDirection = ([plist objectForKey:@"revertDirection"] ? YES : NO);
-    side = [plist intForKey:@"side"];
-    dippingDepth = [plist floatForKey:@"dippingDepth"];
-    approachAngle = [plist floatForKey:@"approachAngle"];
-    flatness = [plist floatForKey:@"flatness"];
-    fillOverlap = [plist floatForKey:@"fillOverlap"];
-    fillDirection = [plist floatForKey:@"fillDirection"];
+    side            = [plist intForKey:@"side"];
+    dippingDepth    = [plist floatForKey:@"dippingDepth"];
+    approachAngle   = [plist floatForKey:@"approachAngle"];
+    flatness        = [plist floatForKey:@"flatness"];
+    fillOverlap     = [plist floatForKey:@"fillOverlap"];
+    fillDirection   = [plist floatForKey:@"fillDirection"];
     if ([plist objectForKey:@"minLineLength"])	// Cenon 3.30 - 3.31
         fillOverlap = 1.0 - fillOverlap;
-    inlay = ([plist objectForKey:@"inlay"] ? YES : NO);
+    inlay           = ([plist objectForKey:@"inlay"] ? YES : NO);
 
-    stepwise = ([plist objectForKey:@"stepwise"] ? YES : NO);
-    numSteps = [plist intForKey:@"numSteps"];
-    step[0] = [plist floatForKey:@"step0"];
-    step[1] = [plist floatForKey:@"step1"];
-    step[2] = [plist floatForKey:@"step2"];
+    stepwise        = ([plist objectForKey:@"stepwise"] ? YES : NO);
+    numSteps        = [plist intForKey:@"numSteps"];
+    step[0]         = [plist floatForKey:@"step0"];
+    step[1]         = [plist floatForKey:@"step1"];
+    step[2]         = [plist floatForKey:@"step2"];
     calcSteps.count = 0;
 
-    settleBefore = ([plist objectForKey:@"settleBefore"] ? YES : NO);
-    settle = [plist floatForKey:@"settle"];
+    settleBefore    = ([plist objectForKey:@"settleBefore"] ? YES : NO);
+    settle          = [plist floatForKey:@"settle"];
+
+    invisible       = ([plist objectForKey:@"invisible"] ? YES : NO);
 
     return self;
 }
@@ -731,8 +771,9 @@
 {
     [string release];
     [list release]; list = nil;
-    [performanceMap removeAllObjects];	// remove references from graphics
+    [performanceMap removeAllObjects];  // remove references from graphics
     [performanceMap release];
+    [layerDict release];
 
     [toolString release];
 
