@@ -1,13 +1,14 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2011 by vhf interservice GmbH
+ * Copyright (C) 1996-2012 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2011-02-27 (-init: initialize group.handle, state.color, blockScanner, rable, vivibleList)
+ * modified: 2012-02-04 (-scanGroup: transform coordinates from OCS to WCS, if necessary)
+ *           2011-02-27 (-init: initialize group.handle, state.color, blockScanner, rable, vivibleList)
  *           2010-10-31 (buildArc(): dx, dy calced with double)
- *           2010-09-11 (-interpret: try UTF8 and Latin encodings to make GNUstep happy)
+ *           2010-09-11 (-importDXF: try UTF8 and Latin encodings to make GNUstep happy)
  *           2010-06-11 (buildArc(): 360.0 - acos(cn), was 180.0 - )
  *           2010-04-03 (-getArc: negative 360 deg arcs work now, buildArc(): dx == 0 case)
  *           2007-01-30 (close PolyLine: check for start == end point)
@@ -79,7 +80,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 - (BOOL)getCircle:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
 - (BOOL)getArc:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle;
 - (BOOL)getText :(NSScanner*)scanner mtext:(BOOL)mext :(NSString**)string :(float*)angle :(NSPoint*)origin :(float*)size :(float*)ar :(int*)alignment;
-- (BOOL)getInsert:(NSScanner*)scanner :cList;
+- (BOOL)getInsert:(NSScanner*)scanner :(id)cList;
 - (void)updateBounds:(NSPoint)p;
 
 - (void)addText:(NSString*)text :(NSString*)font :(float)angle :(float)size :(float)ar at:(NSPoint)p 	toLayer:(NSString*)layerName;	// old
@@ -1291,13 +1292,13 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
  * parameter: scanner (the dxf data)
  *            cList
  */
-- (BOOL)getInsert:(NSScanner*)scanner :cList
-{   NSPoint		point;				/* position of the text */
-    NSString		*name, *string;
-    float		angle;
-    NSScanner		*myBlockScanner = nil;
-    int			location;
-    NSAutoreleasePool	*pool = [[NSAutoreleasePool alloc] init];
+- (BOOL)getInsert:(NSScanner*)scanner :(id)cList
+{   NSPoint             point;				/* position of the text */
+    NSString            *name, *string;
+    float               angle;
+    NSScanner           *myBlockScanner = nil;
+    int                 location;
+    NSAutoreleasePool   *pool = [[NSAutoreleasePool alloc] init];
 
     [self scanGroup:scanner];
     if ( [visibleList count] && ![visibleList containsObject:group.handle])
@@ -1404,6 +1405,53 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     return color;
 }
 
+/* arbitrary axis algorithm - transform OCS (Object) to WCS (World)
+ * build the cross product
+ *
+ * W        our coordinate             [WCS]
+ * AZ (N)   the extrusion direction    [WCS]
+ * return   the transformed coordinate [OCS]
+ * created: 2012-02-03
+ */
+static V3Point crossProduct(const V3Point a, const V3Point b)
+{   V3Point result;
+
+    result.x = a.y * b.z - a.z * b.y;
+    result.y = a.z * b.x - a.x * b.z;
+    result.z = a.x * b.y - a.y * b.x;
+    return result;
+}
+static V3Point scaleToUnit(V3Point p)
+{   double  len = sqrt(p.x*p.x + p.y*p.y + p.z*p.z);
+    
+    if (len != 0)
+    {   p.x /= len;
+        p.y /= len;
+        p.z /= len;
+    }
+    return p;
+}
+static V3Point transformOCS(V3Point pOCS, V3Point AZ)
+{   V3Point AX, AY, p;
+
+    if ( Abs(AZ.x) < 1.0/64.0 && Abs(AZ.y) < 1.0/64.0 )
+        AX = crossProduct(V3MakePoint(0.0, 1.0, 0.0), AZ);   // transform using unit vector Y
+    else
+        AX = crossProduct(V3MakePoint(0.0, 0.0, 1.0), AZ);   // transform using unit vector Z
+    AX = scaleToUnit( AX );  // scale vector to unit (length = 1.0)
+    AY = crossProduct(AZ, AX);
+    AY = scaleToUnit( AY );
+
+    /* these are direction cosines, we have to rotate each axis around this */
+    {   double x = pOCS.x, y = pOCS.y, z = pOCS.z;
+
+        p.x = x * AX.x + y * AY.x + z * AZ.x;
+        p.y = x * AX.y + y * AY.y + z * AZ.y;
+        p.z = x * AX.z + y * AY.z + z * AZ.z;
+    }
+    return p;
+}
+
 /* get group entries
  * scanner starts at id line of first entry
  * scanner scans until it reaches the next group
@@ -1411,7 +1459,8 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
  * modified: 2009-02-06 (read in extrusion direction)
  */
 - (BOOL)scanGroup:(NSScanner*)scanner
-{
+{   BOOL    exitLoop = NO;
+
     [group.text release]; group.text = nil;
     [group.name release]; group.name = nil;
     [group.layer release]; group.layer = nil;
@@ -1426,7 +1475,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     group.x3 = group.y3 = 0.0;
     group.a = 0.0;
     group.flags = group.genFlags = 0;
-    group.extX = group.extY = 0.0; group.extZ = -1.0;
+    group.extX = group.extY = 0.0; group.extZ = 1.0;
 
     while ( ![scanner isAtEnd] )
     {	int	i, location = [scanner scanLocation];
@@ -1564,12 +1613,30 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
                 break;
             case ID_GROUP:
                 [scanner setScanLocation:location];
-                return YES;
+                exitLoop = YES;
+                break;
             default:
                 break;
         }
+        if ( exitLoop )
+            break;
         gotoNextLine(scanner);	// goto id line
     }
+
+    /* transform coordinates from OCS to WCS (only if extrusion direction tells us so) */
+    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
+    {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
+
+        p = transformOCS(V3MakePoint(group.x0, group.y0, 0.0), N);
+        group.x0 = p.x; group.y0 = p.y;
+        p = transformOCS(V3MakePoint(group.x1, group.y1, 0.0), N);
+        group.x1 = p.x; group.y1 = p.y;
+        p = transformOCS(V3MakePoint(group.x2, group.y2, 0.0), N);
+        group.x2 = p.x; group.y2 = p.y;
+        p = transformOCS(V3MakePoint(group.x3, group.y3, 0.0), N);
+        group.x3 = p.x; group.y3 = p.y;
+    }
+
     return YES;
 }
 
